@logo-height: 28px;

.jpy-logo {
  height: @logo-height;
  margin-top: (@navbar-height - @logo-height) / 2;
}

#header {
  border-bottom: 1px solid #e7e7e7;
}

.hidden {
  display: none;
}

#progress-log {
  margin-top: 8px;
}

.progress-log-event {
  border-top: 1px solid #e7e7e7;
  padding: 8px;
}

// hover-highlight on log events?
// .progress-log-event:hover {
//   background: rgba(66, 165, 245, 0.2);
// }
