{% extends "page.html" %}
{% if announcement_home %}
  {% set announcement = announcement_home %}
{% endif %}

{% block main %}

<div class="container">
  <div class="row">
    <div class="text-center">
      {% if user.running %}
      <a id="stop" role="button" class="btn btn-lg btn-danger">Stop My Server</a>
      {% endif %}
      <a id="start" role="button" class="btn btn-lg btn-primary" href="{{ url }}">
      {% if not user.active %}
      Start
      {% endif %}
        My Server
      </a>
    </div>
  </div>
</div>

{% endblock %}

{% block script %}
{{ super() }}
<script type="text/javascript">
require(["home"]);
</script>
{% endblock %}
