{% extends "page.html" %}
{% if announcement_spawn %}
  {% set announcement = announcement_spawn %}
{% endif %}

{% block main %}

<div class="container">
  {% block heading %}
  <div class="row text-center">
    <h1>Spawner Options</h1>
  </div>
  {% endblock %}
  <div class="row col-sm-offset-2 col-sm-8">
    {% if for_user and user.name != for_user.name -%}
      <p>Spawning server for {{ for_user.name }}</p>
    {% endif -%}
    {% if error_message -%}
      <p class="spawn-error-msg text-danger">
        Error: {{error_message}}
      </p>
    {% endif %}
    <form enctype="multipart/form-data" id="spawn_form" action="{{url}}" method="post" role="form">
      {{spawner_options_form | safe}}
      <br>
      <input type="submit" value="Spawn" class="btn btn-jupyter form-control">
    </form>
  </div>
</div>

{% endblock %}
